# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1448, 843)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.toolBox = QtWidgets.QToolBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBox.sizePolicy().hasHeightForWidth())
        self.toolBox.setSizePolicy(sizePolicy)
        self.toolBox.setMinimumSize(QtCore.QSize(210, 0))
        self.toolBox.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.toolBox.setObjectName("toolBox")
        self.fileIo = QtWidgets.QWidget()
        self.fileIo.setGeometry(QtCore.QRect(0, -125, 200, 840))
        self.fileIo.setObjectName("fileIo")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.fileIo)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.label_8 = QtWidgets.QLabel(self.fileIo)
        self.label_8.setWordWrap(True)
        self.label_8.setObjectName("label_8")
        self.verticalLayout.addWidget(self.label_8)
        self.labelCurrentFile = QtWidgets.QLabel(self.fileIo)
        self.labelCurrentFile.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelCurrentFile.sizePolicy().hasHeightForWidth())
        self.labelCurrentFile.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setItalic(True)
        self.labelCurrentFile.setFont(font)
        self.labelCurrentFile.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.labelCurrentFile.setWordWrap(True)
        self.labelCurrentFile.setOpenExternalLinks(True)
        self.labelCurrentFile.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse)
        self.labelCurrentFile.setObjectName("labelCurrentFile")
        self.verticalLayout.addWidget(self.labelCurrentFile)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.label_5 = QtWidgets.QLabel(self.fileIo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setScaledContents(False)
        self.label_5.setAlignment(QtCore.Qt.AlignCenter)
        self.label_5.setObjectName("label_5")
        self.verticalLayout.addWidget(self.label_5)
        self.pushButtonNewFromTemplate = QtWidgets.QPushButton(self.fileIo)
        self.pushButtonNewFromTemplate.setObjectName("pushButtonNewFromTemplate")
        self.verticalLayout.addWidget(self.pushButtonNewFromTemplate)
        self.pushButtonLoadFile = QtWidgets.QPushButton(self.fileIo)
        self.pushButtonLoadFile.setObjectName("pushButtonLoadFile")
        self.verticalLayout.addWidget(self.pushButtonLoadFile)
        self.pushButtonSaveToFile = QtWidgets.QPushButton(self.fileIo)
        self.pushButtonSaveToFile.setObjectName("pushButtonSaveToFile")
        self.verticalLayout.addWidget(self.pushButtonSaveToFile)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.label_11 = QtWidgets.QLabel(self.fileIo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_11.setFont(font)
        self.label_11.setAlignment(QtCore.Qt.AlignCenter)
        self.label_11.setObjectName("label_11")
        self.verticalLayout.addWidget(self.label_11)
        self.pushButtonExportBranchTxt = QtWidgets.QPushButton(self.fileIo)
        self.pushButtonExportBranchTxt.setObjectName("pushButtonExportBranchTxt")
        self.verticalLayout.addWidget(self.pushButtonExportBranchTxt)
        self.pushButtonExportTreeTxt = QtWidgets.QPushButton(self.fileIo)
        self.pushButtonExportTreeTxt.setObjectName("pushButtonExportTreeTxt")
        self.verticalLayout.addWidget(self.pushButtonExportTreeTxt)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem3)
        self.label_9 = QtWidgets.QLabel(self.fileIo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName("label_9")
        self.verticalLayout.addWidget(self.label_9)
        self.cboxTheme = QtWidgets.QComboBox(self.fileIo)
        self.cboxTheme.setObjectName("cboxTheme")
        self.verticalLayout.addWidget(self.cboxTheme)
        self.cboxColours = QtWidgets.QComboBox(self.fileIo)
        self.cboxColours.setObjectName("cboxColours")
        self.verticalLayout.addWidget(self.cboxColours)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem5)
        self.label_10 = QtWidgets.QLabel(self.fileIo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_10.setFont(font)
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setObjectName("label_10")
        self.verticalLayout.addWidget(self.label_10)
        self.label_7 = QtWidgets.QLabel(self.fileIo)
        self.label_7.setObjectName("label_7")
        self.verticalLayout.addWidget(self.label_7)
        self.toolBox.addItem(self.fileIo, "")
        self.pageTree = QtWidgets.QWidget()
        self.pageTree.setGeometry(QtCore.QRect(0, 0, 200, 716))
        self.pageTree.setMinimumSize(QtCore.QSize(200, 716))
        self.pageTree.setObjectName("pageTree")
        self.formLayout = QtWidgets.QFormLayout(self.pageTree)
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.pageTree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.SpanningRole, self.label)
        self.pushButtonNewChild = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonNewChild.setObjectName("pushButtonNewChild")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.SpanningRole, self.pushButtonNewChild)
        self.pushButtonNewSibling = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonNewSibling.setObjectName("pushButtonNewSibling")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.SpanningRole, self.pushButtonNewSibling)
        self.pushButtonNewParent = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonNewParent.setObjectName("pushButtonNewParent")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.SpanningRole, self.pushButtonNewParent)
        self.label_2 = QtWidgets.QLabel(self.pageTree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.SpanningRole, self.label_2)
        self.pushButtonCopyNodeChild = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonCopyNodeChild.setObjectName("pushButtonCopyNodeChild")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.SpanningRole, self.pushButtonCopyNodeChild)
        self.pushButtonCopyNodeSibling = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonCopyNodeSibling.setObjectName("pushButtonCopyNodeSibling")
        self.formLayout.setWidget(9, QtWidgets.QFormLayout.SpanningRole, self.pushButtonCopyNodeSibling)
        self.pushButtonCopyNodeParent = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonCopyNodeParent.setObjectName("pushButtonCopyNodeParent")
        self.formLayout.setWidget(10, QtWidgets.QFormLayout.SpanningRole, self.pushButtonCopyNodeParent)
        self.label_3 = QtWidgets.QLabel(self.pageTree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(12, QtWidgets.QFormLayout.SpanningRole, self.label_3)
        self.label_6 = QtWidgets.QLabel(self.pageTree)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.pushButtonCopyBranchSibling = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonCopyBranchSibling.setObjectName("pushButtonCopyBranchSibling")
        self.formLayout.setWidget(15, QtWidgets.QFormLayout.SpanningRole, self.pushButtonCopyBranchSibling)
        self.pushButtonCopyChildrenSiblings = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonCopyChildrenSiblings.setObjectName("pushButtonCopyChildrenSiblings")
        self.formLayout.setWidget(16, QtWidgets.QFormLayout.SpanningRole, self.pushButtonCopyChildrenSiblings)
        self.label_4 = QtWidgets.QLabel(self.pageTree)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(18, QtWidgets.QFormLayout.SpanningRole, self.label_4)
        self.pushButtonRemove = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonRemove.setObjectName("pushButtonRemove")
        self.formLayout.setWidget(19, QtWidgets.QFormLayout.SpanningRole, self.pushButtonRemove)
        self.pushButtonDelete = QtWidgets.QPushButton(self.pageTree)
        self.pushButtonDelete.setObjectName("pushButtonDelete")
        self.formLayout.setWidget(20, QtWidgets.QFormLayout.SpanningRole, self.pushButtonDelete)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(1, QtWidgets.QFormLayout.SpanningRole, spacerItem6)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(6, QtWidgets.QFormLayout.SpanningRole, spacerItem7)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(11, QtWidgets.QFormLayout.SpanningRole, spacerItem8)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(17, QtWidgets.QFormLayout.SpanningRole, spacerItem9)
        self.comboBox = QtWidgets.QComboBox(self.pageTree)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox.sizePolicy().hasHeightForWidth())
        self.comboBox.setSizePolicy(sizePolicy)
        self.comboBox.setObjectName("comboBox")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.formLayout.setWidget(13, QtWidgets.QFormLayout.FieldRole, self.comboBox)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(22, QtWidgets.QFormLayout.SpanningRole, spacerItem10)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(21, QtWidgets.QFormLayout.SpanningRole, spacerItem11)
        self.toolBox.addItem(self.pageTree, "")
        self.pageGlobal = QtWidgets.QWidget()
        self.pageGlobal.setGeometry(QtCore.QRect(0, 0, 96, 26))
        self.pageGlobal.setObjectName("pageGlobal")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.pageGlobal)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem12, 0, 0, 1, 1)
        self.toolBox.addItem(self.pageGlobal, "")
        self.horizontalLayout.addWidget(self.toolBox)
        self.tableWidget = QtWidgets.QTableWidget(self.centralwidget)
        self.tableWidget.setMidLineWidth(0)
        self.tableWidget.setShowGrid(False)
        self.tableWidget.setWordWrap(True)
        self.tableWidget.setRowCount(20)
        self.tableWidget.setColumnCount(5)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.horizontalHeader().setVisible(False)
        self.tableWidget.horizontalHeader().setCascadingSectionResizes(False)
        self.tableWidget.horizontalHeader().setDefaultSectionSize(10)
        self.tableWidget.horizontalHeader().setMinimumSectionSize(10)
        self.tableWidget.horizontalHeader().setSortIndicatorShown(True)
        self.tableWidget.horizontalHeader().setStretchLastSection(False)
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.verticalHeader().setCascadingSectionResizes(True)
        self.horizontalLayout.addWidget(self.tableWidget)
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setDocumentMode(False)
        self.tabWidget.setObjectName("tabWidget")
        self.horizontalLayout.addWidget(self.tabWidget)
        self.horizontalLayout.setStretch(0, 1)
        self.horizontalLayout.setStretch(1, 3)
        self.horizontalLayout.setStretch(2, 5)
        MainWindow.setCentralWidget(self.centralwidget)
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")

        self.retranslateUi(MainWindow)
        self.toolBox.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(-1)
        self.actionQuit.triggered.connect(MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Tree Time"))
        self.label_8.setText(_translate("MainWindow", "Connected to Data File:"))
        self.labelCurrentFile.setText(_translate("MainWindow", "[current file name]"))
        self.label_5.setText(_translate("MainWindow", "Load and Save"))
        self.pushButtonNewFromTemplate.setText(_translate("MainWindow", "New from Template"))
        self.pushButtonLoadFile.setText(_translate("MainWindow", "Load File"))
        self.pushButtonSaveToFile.setText(_translate("MainWindow", "Save As"))
        self.label_11.setText(_translate("MainWindow", "Export"))
        self.pushButtonExportBranchTxt.setText(_translate("MainWindow", "Export Branch to Textfile"))
        self.pushButtonExportTreeTxt.setText(_translate("MainWindow", "Export Tree to Textfile"))
        self.label_9.setText(_translate("MainWindow", "Look and Feel"))
        self.label_10.setText(_translate("MainWindow", "About"))
        self.label_7.setText(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">     TreeTime</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">     Version 2020.07</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">     New versions,<br />     source code and <br />     documentation at <br />     <a href=\"https://github.com/jkanev/treetime\"><span style=\" text-decoration: underline; color:#2980b9;\">github.com/jkanev/treetime</span></a></p></body></html>"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.fileIo), _translate("MainWindow", "File"))
        self.label.setText(_translate("MainWindow", "Insert New Nodes"))
        self.pushButtonNewChild.setText(_translate("MainWindow", "New Child"))
        self.pushButtonNewSibling.setText(_translate("MainWindow", "New Sibling"))
        self.pushButtonNewParent.setText(_translate("MainWindow", "New Parent"))
        self.label_2.setText(_translate("MainWindow", "Copy Existing Nodes"))
        self.pushButtonCopyNodeChild.setText(_translate("MainWindow", "Copy Node as Child"))
        self.pushButtonCopyNodeSibling.setText(_translate("MainWindow", "Copy Node as Sibling"))
        self.pushButtonCopyNodeParent.setText(_translate("MainWindow", "Copy Node as Parent"))
        self.label_3.setText(_translate("MainWindow", "Copy Parts of the Tree"))
        self.label_6.setText(_translate("MainWindow", "Depth  "))
        self.pushButtonCopyBranchSibling.setText(_translate("MainWindow", "Copy Branch as Sibling"))
        self.pushButtonCopyChildrenSiblings.setText(_translate("MainWindow", "Copy Children to Siblings"))
        self.label_4.setText(_translate("MainWindow", "Remove Single Nodes"))
        self.pushButtonRemove.setText(_translate("MainWindow", "Remove from this Tree"))
        self.pushButtonDelete.setText(_translate("MainWindow", "Delete Item"))
        self.comboBox.setItemText(0, _translate("MainWindow", "all levels"))
        self.comboBox.setItemText(1, _translate("MainWindow", "1 level"))
        self.comboBox.setItemText(2, _translate("MainWindow", "2 levels"))
        self.comboBox.setItemText(3, _translate("MainWindow", "3 levels"))
        self.comboBox.setItemText(4, _translate("MainWindow", "4 levels"))
        self.comboBox.setItemText(5, _translate("MainWindow", "5 levels"))
        self.comboBox.setItemText(6, _translate("MainWindow", "6 levels"))
        self.comboBox.setItemText(7, _translate("MainWindow", "7 levels"))
        self.comboBox.setItemText(8, _translate("MainWindow", "8 levels"))
        self.comboBox.setItemText(9, _translate("MainWindow", "9 levels"))
        self.comboBox.setItemText(10, _translate("MainWindow", "10 levels"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.pageTree), _translate("MainWindow", "Edit (this Tree)"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.pageGlobal), _translate("MainWindow", "Edit (all Trees)"))
        self.tableWidget.setSortingEnabled(False)
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
